@@ -196,6 +196,8 @@ class StepCounter:
         Y_ = np.zeros_like(W_, dtype='float')
         Y_[W_] = batch_count_peaks(X_[W_], self.sample_rate, self.lowpass_hz, self.find_peaks_params)
 
+        # TODO: should we zero out < steptol windows?
+
         Y = np.full(len(X), fill_value=np.nan)
         Y[whr_ok] = Y_
 
@@ -212,15 +214,18 @@ class StepCounter:
             """ Process the chunk. Apply padding if length is not enough. """
             n = len(chunk)
             x = chunk[['x', 'y', 'z']].to_numpy()
-            if n > self.window_len:
+            if n == self.window_len:
+                x = x
+            elif n > self.window_len:
                 x = x[:self.window_len]
-            if n < self.window_len:
+            elif n < self.window_len and n > self.window_len / 2:
                 m = self.window_len - n
                 x = np.pad(x, ((0, m), (0, 0)), mode='wrap')
+            else:
+                x = np.full((self.window_len, 3), fill_value=np.nan)
             return x
 
-        X, T = make_windows(data, self.window_sec, fn=fn, return_index=True)
-        X = np.asarray(X)
+        X, T = make_windows(data, self.window_sec, fn=fn, return_index=True, verbose=self.verbose)
         Y = self.predict(X, **kwargs)
         Y = pd.Series(Y, index=T)
         return Y
@@ -269,7 +274,7 @@ class WalkDetectorRF:
 
     def fit(self, X, Y, groups=None):
 
-        X_feats = batch_extract_features(X, self.sample_rate, n_jobs=self.n_jobs)
+        X_feats = batch_extract_features(X, self.sample_rate, n_jobs=self.n_jobs, verbose=self.verbose)
 
         whr_ok = ~(np.isnan(X_feats).any(1))
         X_feats = X_feats[whr_ok]
@@ -321,7 +326,7 @@ class WalkDetectorRF:
         return self
 
     def predict(self, X, groups=None):
-        X_feats = batch_extract_features(X, self.sample_rate, n_jobs=self.n_jobs)
+        X_feats = batch_extract_features(X, self.sample_rate, n_jobs=self.n_jobs, verbose=self.verbose)
         whr_ok = ~(np.isnan(X_feats).any(1))
         W = np.zeros(len(X), dtype='int')  # nan defaults to non-walk
         W[whr_ok] = (self.clf.predict_proba(X_feats[whr_ok])[:, 1] > self.thresh).astype('int')
@@ -421,9 +426,9 @@ class WalkDetectorSSL:
         dataset = sslmodel.NormalDataset(X, name='prediction')
         dataloader = DataLoader(
             dataset,
-            batch_size=self.batch_size,
+            batch_size=512,
             shuffle=False,
-            num_workers=1,
+            num_workers=0,
         )
 
         model = sslmodel.get_sslnet(tag=self.repo_tag, pretrained=False)
@@ -437,22 +442,26 @@ class WalkDetectorSSL:
         return y_pred
 
 
-def make_windows(data, window_sec, fn=None, return_index=False):
+def make_windows(data, window_sec, fn=None, return_index=False, verbose=True):
     """ Split data into windows """
 
+    if verbose:
+        print("Defining windows...")
+
     if fn is None:
         def fn(x):
             return x
 
-    X = [fn(x) for _, x in data.resample(f"{window_sec}s", origin="start")]
+    X, T = [], []
+    for t, x in tqdm(data.resample(f"{window_sec}s", origin="start"), mininterval=5, disable=not verbose):
+        x = fn(x)
+        X.append(x)
+        T.append(t)
+
+    X = np.asarray(X)
 
     if return_index:
-        T = (
-            data.index
-            .to_series()
-            .resample(f"{window_sec}s", origin="start")
-            .first()
-        )
+        T = pd.DatetimeIndex(T, name=data.index.name)
         return X, T
 
     return X
@@ -570,9 +579,12 @@ def get_cv_scores(yt, yp, cv_test_idxs, sample_weight=None, scorer_type='classif
 def batch_extract_features(X, sample_rate, to_numpy=True, n_jobs=1, verbose=False):
     """ Extract features for a list or array of windows """
 
+    if verbose:
+        print("Extracting features...")
+
     X_feats = Parallel(n_jobs=n_jobs)(
         delayed(features.extract_features)(x, sample_rate)
-        for x in tqdm(X, disable=not verbose)
+        for x in tqdm(X, mininterval=5, disable=not verbose)
     )
     X_feats = pd.DataFrame(X_feats)
 
